        include "doorsos.h"
        include "graphlib.h"
        include "userlib.h"
        include "hufflib.h"

        xdef _main
        xdef _comment
        xdef _ti89
		
kbd_arrows	EQU		%111111110
kbd_column5	EQU		%111111101
kbd_column4	EQU		%111111011
kbd_column3	EQU		%111110111
kbd_column2	EQU		%111101111
kbd_column1	EQU		%111011111
kbd_exitkey	EQU		%110111111

dir_left	EQU		0
dir_up		EQU		1
dir_right	EQU		2
dir_down	EQU		3

_main:
	bsr		WaitKBD

	move.l	#LCD_MEM,a0
	bsr		ClearScreen
		
	bsr		DoProgramInit			; do program initialization stuff
	
	tst.w	teacherkeyload			; test if you should play the saved game
	bne		Game_Play				; if so, skip the menu
Game_Menu:
	bsr		DoGameMenu				; show the title and let user pick an option
Game_Menu_Skip:
	cmp.b	#0,barpos				; PLAY?
	beq		Game_Play
	; --- options should be called directly from the MENU
	bra		Game_Quit				; must have picked QUIT
		
Game_Play:
	bsr		DoPlayGame
	bra		Game_Quit
Game_Quit:		
	bsr		DoProgramShutdown		; restore normal TI state
	bsr		WaitKBD					; done return until all keys are released
	rts
		
; ***************************************************************
; ********************** Do... subroutines **********************
; ***************************************************************

; ************ Initialization and Shutdown

; ** Program Initialization
DoProgramInit:
	move.w  #$700,d0                    ; Disable interrupts, saving old in mask in d0
    trap    #1
	bclr.b	#2,($600001)				; disable memory protection
	move.l 	($64),Old_Int1				; save old int1 address
	move.l	#New_Int1,($64)				; set int1 to New_Int1
	bset.b	#2,($600001)				; reenable memory protection
	trap	#1							; restore old interrupt mask (from d0)
	
	rts
	
; ** Program Shutdown
DoProgramShutdown:
	move.w  #$700,d0                    ; Disable interrupts, saving old in mask in d0
    trap    #1
	bclr.b	#2,($600001)				; disable memory protection
	move.l	Old_Int1,($64)				; set int1 back
	bset.b	#2,($600001)				; reenable memory protection
	trap	#1							; restore old interrupt mask (from d0)
	
	rts


; ************ Play/Options/Quit menu (GRAYSCALE) routines

; ** Main routine
DoGameMenu:
	move.b	#0,barpos						; set bar position to 1st option (play)
	
	move.l 	#titlePicBits,a0				; extract compressed [titlePicBits] into [screensave0,1,2]
    move.l  #screensave0,a1
    move.w  #0,d3
    jsr hufflib::extract
			
    jsr graphlib::gray4							; initialize and display grayscale screen
	
DoGameMenu_Reshow:		
	move.l	#screensave0,a0					; copy the different planes from extracted data
	move.l	graphlib::plane1,a1
	bsr		CopyToGrayPlane
	move.l	#screensave1,a0
	move.l	graphlib::plane0,a1
	bsr		CopyToGrayPlane

	move.b	#0,KST_UP					; reset key states
	move.b	#0,KST_DOWN
				
DoGameMenu_Loop1:
	bsr		DrawBar						; draw gray bar to signify which selection is chosen
DoGameMenu_Loop2:
	move.w	#kbd_arrows,d0				; check keys (arrows)
	bsr		ReadKBD
	btst.b	#4,d0						; test for 2ND key
	beq		DoGameMenu_2ND
	btst.b	#0,d0						; test for UP key
	beq		DoGameMenu_UP
	move.b	#0,KST_UP					; reset key state
	btst.b	#2,d0						; test for DOWN key
	beq		DoGameMenu_DOWN
	move.b	#0,KST_DOWN					; reset key state
	move.w	#kbd_column5,d0				; check keys (column5)
	bsr		ReadKBD
	btst.b	#0,d0						; test for ENTER key
	beq		DoGameMenu_2ND
			
	move.w	#kbd_exitkey,d0				; check keys (exitkey)
	bsr		ReadKBD
	btst.b	#0,d0						; test for EXIT key
	beq		DoGameMenu_EXIT

	bra     DoGameMenu_Loop2			; if nothing useful pressed, loop
		
DoGameMenu_2ND:							; 2ND key or ENTER has been pressed, return from sub
	cmp.b	#1,barpos					; if its options, then do the options menu
	beq		DoGameMenu_Options
	jsr graphlib::gray2					; otherwise return from sub
    rts

DoGameMenu_EXIT							; EXIT key has been pressed, set to QUIT and return
	move.b	#2,barpos
	jsr	graphlib::gray2
    rts
	
DoGameMenu_UP:
	tst.b	KST_UP						; check to see if it has been repressed or is still down
	bne		DoGameMenu_Loop2			; if its just still down, loop
	move.b	#1,KST_UP					; set keystate
	
    move.b  barpos,d0               ; check to see if [barpos].b=0
	tst.b	d0
	beq		DoGameMenu_UP_Loop1			; if so, jump
	sub.b	#1,barpos					; if not, just subtract 1
	bra		DoGameMenu_Loop1			; and go back into loop, redrawing the bar
DoGameMenu_UP_Loop1:
	move.b	#2,barpos					; if so, set it to 2
	bra		DoGameMenu_Loop1			; and go back into loop, redrawing the bar

DoGameMenu_DOWN:
	tst.b	KST_DOWN					; check to see if it has been repressed or is still down
	bne		DoGameMenu_Loop2			; if its just still down, loop
	move.b	#1,KST_DOWN					; set keystate
	
    move.b  barpos,d0               ; check to see if [barpos].b=2
	cmp.b	#2,d0
	beq		DoGameMenu_DOWN_Loop1		; if so, jump
	add.b	#1,barpos					; if not, just add 1
	bra		DoGameMenu_Loop1			; and go back into loop, redrawing the bar
DoGameMenu_DOWN_Loop1:
	move.b	#0,barpos					; if so, set it to 0
	bra		DoGameMenu_Loop1			; and go back into loop, redrawing the bar


DoGameMenu_Options:
	move.b	#0,bigbarpos				; set option bar to 1st option

	bsr		SetFontSMALL

	move.l	graphlib::plane0,a0			; Clear the bottom portion of the screen
	add.l	#30*65,a0
	move.w	#32,d0
	bsr		ClearLines
	
	move.w	#4,-(a7)					; write the info stuff at the bottom of the screen
	pea		STR_info1
	move.w	#94,-(a7)
	move.w	#2,-(a7)
	jsr		doorsos::DrawStrXY
	add.l	#10,a7
	
	bsr		DoGameMenu_Options_Speed
	bsr		DoGameMenu_Options_ScreenWrap
	bsr		DoGameMenu_Options_ControlStyle

	move.w	#4,-(a7)					; "EXIT"
	pea		STR_option4
	move.w	#84,-(a7)
	move.w	#74,-(a7)
	jsr		doorsos::DrawStrXY
	add.l	#10,a7
	
DoGameMenu_Options_RedoBar:
	move.l	graphlib::plane0,a0			; copy bottom part of plane0 to plane1 (erase bar)
	add.l	#62*30,a0
	move.l	graphlib::plane1,a1
	add.l	#62*30,a1
	move.w	#37,d0
	bsr		CopyLines
	
	bsr		DrawBigBar

DoGameMenu_Options_KeyLoop:
	move.w	#kbd_arrows,d0				; check keys (arrows)
	bsr		ReadKBD
	btst.b	#4,d0						; test for 2ND key
	beq		DoGameMenu_Options_2ND
	btst.b	#0,d0						; test for UP key
	beq		DoGameMenu_Options_UP
	move.b	#0,KST_UP					; reset key state
	btst.b	#2,d0						; test for DOWN key
	beq		DoGameMenu_Options_DOWN
	move.b	#0,KST_DOWN					; reset key state
	btst.b	#1,d0						; test for LEFT key
	beq		DoGameMenu_Options_LEFT
	move.b	#0,KST_LEFT					; reset key state
	btst.b	#3,d0						; test for RIGHT key
	beq		DoGameMenu_Options_RIGHT
	move.b	#0,KST_RIGHT				; reset key state
	move.w	#kbd_column5,d0				; check keys (column5)
	bsr		ReadKBD
	btst.b	#0,d0						; test for ENTER key
	beq		DoGameMenu_Options_2ND
	move.b	#0,KST_2ND
			
	move.w	#kbd_exitkey,d0				; check keys (exitkey)
	bsr		ReadKBD
	btst.b	#0,d0						; test for ESC key
	beq		DoGameMenu_Options_ESC

	bra     DoGameMenu_Options_KeyLoop  ; if nothing useful pressed, loop

DoGameMenu_Options_UP:
	tst.b	KST_UP						; test if the key has been released since last call
	bne		DoGameMenu_Options_KeyLoop	; if so loop
	move.b	#1,KST_UP					; else, set keystate

	add.b	#3,bigbarpos				; bigbarpos=(bigbarpos+3) mod 4
	move.b	bigbarpos,d0
	lsr.w	#2,d0
	lsl.w	#2,d0
	sub.b	d0,bigbarpos
	
	bra 	DoGameMenu_Options_RedoBar  ; return

DoGameMenu_Options_DOWN:
	tst.b	KST_DOWN					; test if the key has been released since last call
	bne		DoGameMenu_Options_KeyLoop  ; if so loop
	move.b	#1,KST_DOWN					; else, set keystate

	add.b	#1,bigbarpos				; bigbarpos=(bigbarpos+1) mod 4
	move.b	bigbarpos,d0
	lsr.w	#2,d0
	lsl.w	#2,d0
	sub.b	d0,bigbarpos
	
	bra 	DoGameMenu_Options_RedoBar  ; return

DoGameMenu_Options_LEFT:
	tst.b	KST_LEFT					; test if the key has been released since last call
	bne		DoGameMenu_Options_KeyLoop  ; if so loop
	move.b	#1,KST_LEFT					; else set key state
	
	cmp.b	#3,bigbarpos				; check where the bar is and do whats necessary
	beq		DoGameMenu_Options_KeyLoop
	cmp.b	#0,bigbarpos
	beq		DoGameMenu_Options_LEFT_Speed
	cmp.b	#1,bigbarpos
	beq		DoGameMenu_Options_RIGHT_ScreenWrap
	bra		DoGameMenu_Options_LEFT_ControlStyle
	
DoGameMenu_Options_LEFT_Speed:		
	cmp.w	#1,GameSpeed				; check if it is 1 (can't go lower)
	beq		DoGameMenu_Options_KeyLoop  ; if so, go back to loop
	sub.w	#1,GameSpeed				; decrease speed by 1
	bsr		DoGameMenu_Options_Speed	; update speed field
	bra		DoGameMenu_Options_KeyLoop	; and return

DoGameMenu_Options_RIGHT:
	tst.b	KST_RIGHT					; test if the key has been released since last call
	bne		DoGameMenu_Options_KeyLoop
	move.b	#1,KST_RIGHT				; else set key state
	
	cmp.b	#3,bigbarpos				; check where the bar is and do whats necessary
	beq		DoGameMenu_Options_KeyLoop
	cmp.b	#0,bigbarpos
	beq		DoGameMenu_Options_RIGHT_Speed
	cmp.b	#1,bigbarpos
	beq		DoGameMenu_Options_RIGHT_ScreenWrap
		;else, goes into...:
DoGameMenu_Options_RIGHT_ControlStyle:
	tst.w FLAG_FourKey
	bne DoGameMenu_Options_RIGHT_ControlStyle_2
	tst.w FLAG_TwoKey
	bne DoGameMenu_Options_RIGHT_ControlStyle_3
	bra		DoGameMenu_Options_KeyLoop	; loop

DoGameMenu_Options_RIGHT_ControlStyle_2:
	clr.w FLAG_FourKey
	clr.w FLAG_360
	move.w #1,FLAG_TwoKey
	bsr		DoGameMenu_Options_ControlStyle ; update field
	bra		DoGameMenu_Options_KeyLoop	; loop

DoGameMenu_Options_RIGHT_ControlStyle_3:
	clr.w FLAG_FourKey
	clr.w FLAG_TwoKey
	move.w #1,FLAG_360
	bsr		DoGameMenu_Options_ControlStyle ; update field
	bra		DoGameMenu_Options_KeyLoop	; loop

DoGameMenu_Options_LEFT_ControlStyle:
	tst.w FLAG_360
	bne DoGameMenu_Options_LEFT_ControlStyle_2
	tst.w FLAG_TwoKey
	bne DoGameMenu_Options_LEFT_ControlStyle_3
	bra		DoGameMenu_Options_KeyLoop	; loop

DoGameMenu_Options_LEFT_ControlStyle_2:
	clr.w FLAG_FourKey
	clr.w FLAG_360
	move.w #1,FLAG_TwoKey
	bsr		DoGameMenu_Options_ControlStyle ; update field
	bra		DoGameMenu_Options_KeyLoop	; loop

DoGameMenu_Options_LEFT_ControlStyle_3:
	clr.w FLAG_360
	clr.w FLAG_TwoKey
	move.w #1,FLAG_FourKey
	bsr		DoGameMenu_Options_ControlStyle ; update field
	bra		DoGameMenu_Options_KeyLoop	; loop
	
DoGameMenu_Options_RIGHT_Speed:
	cmp.w	#10,GameSpeed				; check if speed=10 (cant go higher)
	beq		DoGameMenu_Options_KeyLoop  ; if so, jump
	add.w	#1,GameSpeed				; else add 1 to speed
	bsr		DoGameMenu_Options_Speed    ; and update
	bra		DoGameMenu_Options_KeyLoop	; and loop
	
DoGameMenu_Options_RIGHT_ScreenWrap:
	move.w	#1,d0						; FLAG_WrapAround=1-FLAG_WrapAround
	sub.w	FLAG_WrapAround(pc),d0
	move.w	d0,FLAG_WrapAround
	
	bsr		DoGameMenu_Options_ScreenWrap ; update
	bra		DoGameMenu_Options_KeyLoop	; loop
	
DoGameMenu_Options_2ND:
	tst.b	KST_2ND							; test if the key has been released since last call
	bne		DoGameMenu_Options_KeyLoop
	move.b	#1,KST_2ND
	
	cmp.b	#3,bigbarpos				; if we're not on EXIT, then treat this as RIGHT
	bne		DoGameMenu_Options_RIGHT

DoGameMenu_Options_ESC:					; time to go back
	bsr		WaitKBD
	bra		DoGameMenu_Reshow	
	
DoGameMenu_Options_Speed:
	move.l	graphlib::plane0,a0			; clear this section
	add.l	#30*63,a0
	move.w	#5,d0
	bsr		ClearLines	

	move.w	#4,-(a7)					; write "SPEED"
	pea		STR_option1
	move.w	#63,-(a7)
	move.w	#18,-(a7)
	jsr		doorsos::DrawStrXY
	add.l	#10,a7
	
	move.w	GameSpeed,-(a7)				; turn GameSpeed (.w) into a string
	pea		STR_mask1
	pea		buffer1
	jsr		doorsos::sprintf
	add.l	#10,a7
	
	cmp.w	#10,GameSpeed				; check if speed=10 (so we can draw more to left)
	beq		DoGameMenu_Options_Speed10  ; if so, jump
	
	move.w	#4,-(a7)					; draw the GameSpeed string for 1-9
	pea		buffer1
	move.w	#63,-(a7)
	move.w	#137,-(a7)
	jsr		doorsos::DrawStrXY
	add.l	#10,a7
	rts									; and return
	
DoGameMenu_Options_Speed10:
	move.w	#4,-(a7)					; draw the GameSpeed string for 10
	pea		buffer1
	move.w	#63,-(a7)
	move.w	#133,-(a7)
	jsr		doorsos::DrawStrXY
	add.l	#10,a7
	rts									; and return

DoGameMenu_Options_ScreenWrap:
	move.l	graphlib::plane0,a0			; clear this section
	add.l	#30*70,a0
	move.w	#5,d0
	bsr		ClearLines	

	move.w	#4,-(a7)					; "SCREEN WRAP"
	pea		STR_option2
	move.w	#70,-(a7)
	move.w	#18,-(a7)
	jsr		doorsos::DrawStrXY
	add.l	#10,a7
	
	tst.w	FLAG_WrapAround
	bne		DoGameMenu_Options_ScreenWrap_ON

	move.w	#4,-(a7)					; "OFF"
	pea		STR_option2_0
	move.w	#70,-(a7)
	move.w	#129,-(a7)
	jsr		doorsos::DrawStrXY
	add.l	#10,a7
	
	rts
DoGameMenu_Options_ScreenWrap_ON:
	move.w	#4,-(a7)					; "ON"
	pea		STR_option2_1
	move.w	#70,-(a7)
	move.w	#132,-(a7)
	jsr		doorsos::DrawStrXY
	add.l	#10,a7

	rts
	
DoGameMenu_Options_ControlStyle:
	move.l	graphlib::plane0,a0			; clear this section
	add.l	#30*77,a0
	move.w	#5,d0
	bsr		ClearLines	
	
	move.w	#4,-(a7)					; "CONTROL STYLE"
	pea		STR_option3
	move.w	#77,-(a7)
	move.w	#18,-(a7)
	jsr		doorsos::DrawStrXY
	add.l	#10,a7
	
	tst.w	FLAG_TwoKey
	beq		DoGameMenu_Options_ControlStyle_ON
	
	move.w	#4,-(a7)					; "2-key"
	pea		STR_option3_0
	move.w	#77,-(a7)
	move.w	#120,-(a7)
	jsr		doorsos::DrawStrXY
	add.l	#10,a7
	
	rts
DoGameMenu_Options_ControlStyle_ON:
	tst.w	FLAG_FourKey
	beq		DoGameMenu_Options_ControlStyle_360

	move.w	#4,-(a7)					; "4-key"
	pea		STR_option3_1
	move.w	#77,-(a7)
	move.w	#120,-(a7)
	jsr		doorsos::DrawStrXY
	add.l	#10,a7
	rts
DoGameMenu_Options_ControlStyle_360:
	move.w	#4,-(a7)					; "360"
	pea		STR_option3_2
	move.w	#77,-(a7)
	move.w	#120,-(a7)
	jsr		doorsos::DrawStrXY
	add.l	#10,a7
	rts


	
; ** Copies from decompressed picture data to grayscale planes
;    INPUT:		a0.l=address of decompressed picture data
;    			a1.l=address of grayscale plane
CopyToGrayPlane:
	move.l	d6,-(a7)
	
	move.b	#100,d6						; 100 vertical lines to copy
	
CopyToGrayPlane_Loop1:
	move.l	(a0)+,(a1)+					; copy 20 bytes
	move.l	(a0)+,(a1)+
	move.l	(a0)+,(a1)+
	move.l	(a0)+,(a1)+
	move.l	(a0)+,(a1)+
	
	add.l	#10,a1						; add 10 bytes to account for missing screen area
	
	sub.b	#1,d6						; loop if more lines are left
	tst.b	d6
	bne		CopyToGrayPlane_Loop1
	
	move.l	(a7)+,d6
	rts
	
; ** Draws the short gray bar in plane 0 using [barpos].b (0 to 2)
		
DrawBar:
	move.l	#screensave0,a0			; restore 1st plane to normal state (erasing bar)
	move.l	graphlib::plane1,a1
	bsr		CopyToGrayPlane
		
	move.l	graphlib::plane1,a0		; copy 1st plane offset into a0.l
	add.l	#30*65+4,a0				; add 30*65 bytes to get to line #65, and skip 4 bytes
	
    move.b  barpos,d0           ; the next few lines add 11*30 bytes to a0 for different
DrawBar_Loop1:							; selections (PLAY, OPTIONS, QUIT)
	tst.b	d0
	beq		DrawBar_Continue
	sub.b	#1,d0
	add.l	#11*30,a0
	bra		DrawBar_Loop1
		
DrawBar_Continue:		
	move.b	#9,d0					; set bar height to 9 pixels
DrawBar_Loop2:
	move.l	#$ffffffff,(a0)+		; copy 12 solid bytes
	move.l	#$ffffffff,(a0)+
	move.l	#$ffffffff,(a0)+
	
	add.l	#18,a0					; add 18 bytes (4 to right side, 10 for missing area,
									; 				and 4 more for next line)
	sub.b	#1,d0					; check to see if more lines are needed
	bne		DrawBar_Loop2			; if so, loop
	rts

; ** Draws the short gray bar in plane 0 using [bigbarpos].b (0 to 3)
		
DrawBigBar:
	move.l	graphlib::plane0,a0 	; restore the lower portion of the screen
	add.l	#62*30,a0
	move.l	graphlib::plane1,a1
	add.l	#62*30,a1
	move.w	#37,d0
	bsr		CopyLines
		
	move.l	graphlib::plane1,a0		; copy 1st plane offset into a0.l
	add.l	#30*62+2,a0				; add 30*62 bytes to get to line #62, and skip 2 bytes
	
    move.b  bigbarpos,d0           ; the next few lines add 11*30 bytes to a0 for different
DrawBigBar_Loop1:							; selections (PLAY, OPTIONS, QUIT)
	tst.b	d0
	beq		DrawBigBar_Continue
	sub.b	#1,d0
	add.l	#7*30,a0
	bra		DrawBigBar_Loop1
		
DrawBigBar_Continue:		
	move.b	#7,d0					; set bar height to 7 pixels
DrawBigBar_Loop2:
	move.l	#$ffffffff,(a0)+		; copy 16 solid bytes
	move.l	#$ffffffff,(a0)+
	move.l	#$ffffffff,(a0)+
	move.l	#$ffffffff,(a0)+
	
	add.l	#14,a0					; add 14 bytes (2 to right side, 10 for missing area,
									; 				and 2 more for next line)
	sub.b	#1,d0					; check to see if more lines are needed
	bne		DrawBigBar_Loop2			; if so, loop
	rts

; ************************************************************************************
; ************************************************************************************
; ************************************************************************************
; ************************************************************************************
; ************************************************************************************
; ************ Playing the actual game
DoPlayGame:
	tst.w	teacherkeyload			; test if last game was quit w/ teacher key
	bne		DoPlayGame_SkipScoreSetting		; if so, jump
	
	move.w	#0,P1_Score				; set up for a new game
	move.w	#0,P2_Score

DoPlayGame_SkipScoreSetting:
	bsr  	SetupGameScreen			; draw the border lines
	bsr		DrawScores				; and the scores

DoPlayGame_MainLoop:
	bsr		GetReadyBox
	cmp.b	#1,d0					; check if ESC was hit
	beq		DoPlayGame_Quit			; if so, jump
	cmp.b	#2,d0					; check if F1 was hit
	beq		DoPlayGame_TeacherExit 	; if so, jump
	
	bsr		PlayRound
	cmp.b	#1,d0					; check if ESC was hit
	beq		DoPlayGame_Quit			; if so, jump
	cmp.b	#2,d0					; check if F1 was hit
	beq		DoPlayGame_TeacherExit 	; if so, jump
	
	bra		DoPlayGame_MainLoop

DoPlayGame_TeacherExit:				; teacher key routine
DoPlayGame_Quit:
	rts		
	
; ** Clear screen, draw border lines
SetupGameScreen:
	move.l	#LCD_MEM,a0				; clear screen
	bsr		ClearScreen
	
	move.w	#0,d0					; draw a border around the whole screen
	move.w	#0,d1
	move.w	#159,d4
	move.w	#99,d5
	jsr		graphlib::frame
	
	move.w	#0,d0					; draw the line between score area and game area
	move.w	#8,d1
	move.w	#159,d2
	move.w	#2,d3
	jsr		graphlib::horiz

    rts 

; ** Draw the players' scores in the little boxes at top
DrawScores: 
	bsr		SetFontSMALL			; use the small font
	
    move.w  P1_Score(pc),d0			; put player 1's score in d0.w
    move.w  #2,d6					; and draw it at (*2*,2)
    bsr     PutIntAtD6
	
	move.w  P2_Score(pc),d0			; put player 2's score in d0.w
    move.w  #137,d6					; and draw it at (*137*,2)
	bsr     PutIntAtD6
	
	rts
	
; ** Draw a 5-digit number, padded with 0's if needed, at (d6.w,2)
;    INPUT:		d0.w=integer to draw
;				d6.w=X coordinate
PutIntAtD6:
    move.w  d0,-(a7)				; turn d0.w into a string in [buffer1]
    pea     STR_mask1				; (="%i")
    pea     buffer1
    jsr     doorsos::sprintf
    add.l   #10,a7

    pea     buffer1					; get the length of [buffer1] in d0.w
    jsr     doorsos::strlen
    add.l   #4,a7

    move.w  #6,d2					; set d2.w to 6-d0.w (number of 0's to add plus 1)
    sub.w   d0,d2
PutIntAtD6_Loop1:
    sub.w   #1,d2					; test if more zeros are needed
    tst.w   d2
    beq     PutIntAtD6_Continue		; if not, jump
	
    pea     buffer1					; "0"+[buffer1] => [buffer2] (pad one zero)
    pea     STR_mask2				; (="0%i")
    pea     buffer2
    jsr     doorsos::sprintf
    add.l   #12,a7
	
    pea     buffer2					; copy [buffer2] to [buffer1]
    pea     buffer1
    jsr     doorsos::sprintf
    add.l   #8,a7
	
    bra     PutIntAtD6_Loop1		; loop
	
PutIntAtD6_Continue:
    move.w  #4,-(a7)				; draw string [buffer1] at (d6.w,2)
    pea     buffer1
    move.w  #2,-(a7)
    move.w  d6,-(a7)
    jsr     doorsos::DrawStrXY
    add.l   #10,a7
	
    rts
	
; ** Show a little window that says "GET READY" and wait for ENTER, ESC, or F1 to be pressed
;    OUTPUT:	d0.b=key pressed (0=ENTER, 1=ESC, 2=F1)
GetReadyBox:
	bsr		SetFontSMALL			; set the font to SMALL
	
	move.w	#4,-(a7)				; draw "GET READY..." in middle of top bar
	pea		STR_GetReady
	move.w	#2,-(a7)
	move.w	#59,-(a7)
	jsr		doorsos::DrawStrXY
	add.l	#10,a7   
	
	bsr		WaitKBD					; wait for all keys to be let go

GetReadyBox_Loop1:					; wait for ENTER, ESC, or F1 to be pressed
	move.w	#kbd_column5,d0			; test for ENTER
	bsr		ReadKBD
	btst.b	#0,d0
	beq		GetReadyBox_ENTER
	
	move.w	#kbd_exitkey,d0			; test for ESC
	bsr		ReadKBD
	btst.b	#0,d0
	beq		GetReadyBox_ESC
	
	move.w	#kbd_column1,d0			; test for F1
	bsr		ReadKBD
	btst.b	#7,d0
	beq		GetReadyBox_F1
	
	bra		GetReadyBox_Loop1		; if nothing, loop
	
GetReadyBox_ENTER:
	move.w	#0,-(a7)				; return 0
	bra 	GetReadyBox_Done
GetReadyBox_ESC:
	move.w	#1,-(a7)				; return 1
	move.w	#0,teacherkeyload		; set that the teacher key wasnt
	bra 	GetReadyBox_Done
GetReadyBox_F1:
	move.w	#2,-(a7)				; return 2
	move.w	#1,teacherkeyload		; set that the teacher key was what quit, so you have to keep
									; playing w/ the same scores next time
	
GetReadyBox_Done:
	bsr  	SetupGameScreen			; redraw the screen (fixes the GET READY msg and CRASH)
	bsr		DrawScores
	
	move.w	(a7)+,d0				; put the return value in d0
	rts
	
; ** Plays a round and tells if ESC or F1 was be pressed
;    OUTPUT:	d0.b=key pressed (0=(round complete), 1=ESC, 2=F1)
PlayRound_360:
 move.w #0,P1_Dir
 move.w #180,P2_Dir
 move.w	#1024,P1_X					; set players to starting positions
 move.w	#39680,P2_X
 move.w	#13824,P1_Y
 move.w	#13824,P2_Y	
 bra PlayRound_MainLoop

PlayRound:
	tst.w	FLAG_360
	bne	PlayRound_360
	move.w	#5-1,P1_X					; set players to starting positions
	move.w	#154+1,P2_X
	move.w	#54,P1_Y
	move.w	#54,P2_Y	
	move.w	#dir_right,P1_Dir		; set starting directions
	move.w	#dir_left,P2_Dir
	
PlayRound_MainLoop:
	bsr		DrawDots				; do the moving and dot-drawing
	tst.w	P1_CrashFlag			; check for crash by player 1
	bne		PlayRound_Crash			; if =1, jump
	tst.w	P2_CrashFlag			; check for crash by player 2
	bne		PlayRound_Crash			; if =1, jump
	
	bsr		GameDelay				; slow the game down considerably...

	move.w	#kbd_arrows,d0			; read the arrows row
	bsr		ReadKBD
	
	btst.w	#0,d0					; check for UP key
	beq		PR_Up					; if pressed, jump
	btst.w	#2,d0					; check for DOWN key
	beq		PR_Down					; if pressed, jump
	btst.w	#1,d0					; check for LEFT key
	beq		PR_Left					; if pressed, jump
	btst.w	#3,d0					; check for RIGHT key
	beq		PR_Right				; if pressed, jump
PlayRound_Cont1:
	btst.w	#4,d0					; check for 2ND key
	beq		PR_2nd					; if pressed, jump

	move.w	#kbd_column1,d0			; read column1
	bsr		ReadKBD
	
	btst.w	#6,d0					; check for HOME key
	beq		PR_Home					; if pressed, jump
	
	move.w	#kbd_arrows,d0
	bsr		ReadKBD
	
	btst.w	#6,d0					; check for DIAMOND key
	beq		PR_Diamond				; if pressed, jump
	btst.w	#7,d0					; check for ALPHA key
	beq		PR_Alpha				; if pressed, jump
	
PlayRound_Cont2:

	move.w	#kbd_column1,d0			; read column1
	bsr		ReadKBD

	btst.w	#7,d0					; check for F1 key
	beq		PlayRound_F1			; if pressed, jump
	
	move.w	#kbd_exitkey,d0			; read for esc key
	bsr		ReadKBD
	
	btst.w	#0,d0					; check for esc key
	beq		PlayRound_Esc			; if pressed, jump	
	
	bra		PlayRound_MainLoop
	
PR_Up:
	tst.w	KST_UP					; check if UP has been released since last call
	bne		PlayRound_Cont1			; if not, jump
	move.w	#1,KST_UP				; else set UP keystate
	
	tst.w	FLAG_FourKey			; check if we're playing in 4key mode
	beq		PR_Up_2Key				; if =0, then jump
	
	cmp.w	#dir_down,P2_Dir		; check if we'd kill ourselves if we did this (by going
									; back on ourselves
	beq		PlayRound_Cont1			; if so, ignore
	
	move.w	#dir_up,P2_Dir			; otherwise, set the direction
	bra		PlayRound_Cont1			; and jump
	
PR_Up_2Key:
	tst.w	FLAG_TwoKey			; check if we're playing in 4key mode
	beq		PR_Up_360				; if =0, then jump
    add.w   #1,P2_Dir				; basically:  P2_Dir=(P2_Dir+1) mod 4
    move.w  P2_Dir,d2
    move.w  P2_Dir,d1
    lsr.w   #2,d2
    lsl.w   #2,d2
    sub.w   d2,d1
    move.w  d1,P2_Dir
PR_Up_360:
	bra		PlayRound_Cont1			; and jump
	
PR_Down:
	tst.w	KST_DOWN				; check if DOWN has been released since last call
	bne		PlayRound_Cont1			; if not, jump
	move.w	#1,KST_DOWN				; else set DOWN keystate
	
	tst.w	FLAG_FourKey			; check if we're playing in 4key mode
	beq		PR_Down_2Key			; if =0, then jump
	
	cmp.w	#dir_up,P2_Dir  		; check if we'd kill ourselves if we did this (by going
									; back on ourselves
	beq		PlayRound_Cont1			; if so, ignore
	
	move.w	#dir_down,P2_Dir		; otherwise, set the direction
	bra		PlayRound_Cont1			; and jump
	
PR_Down_2Key:
	tst.w	FLAG_TwoKey			; check if we're playing in 4key mode
	beq		PR_Down_360				; if =0, then jump
    add.w   #3,P2_Dir				; basically:  P2_Dir=(P2_Dir+3) mod 4
    move.w  P2_Dir,d2
    move.w  P2_Dir,d1
    lsr.w   #2,d2
    lsl.w   #2,d2
    sub.w   d2,d1
    move.w  d1,P2_Dir
PR_Down_360:	
	bra		PlayRound_Cont1			; and jump

PR_Left:
	tst.w	FLAG_360			; check if we're playing in 4key mode
	bne		PR_Left_360				; if =0, then jump
	
	tst.w	KST_LEFT				; check if LEFT has been released since last call
	bne		PlayRound_Cont1			; if not, jump
	move.w	#1,KST_LEFT				; else set LEFT keystate
	
	tst.w	FLAG_TwoKey			; check if we're playing in 4key mode
	bne		PlayRound_Cont1			; if =0, then jump

	cmp.w	#dir_right,P2_Dir  		; check if we'd kill ourselves if we did this (by going
									; back on ourselves
	beq		PlayRound_Cont1			; if so, ignore
	
	move.w	#dir_left,P2_Dir		; otherwise, set the direction
	bra		PlayRound_Cont1			; and jump

PR_Left_360:
 add.w #10,P2_Dir
 cmp #360,P2_Dir
 blt PlayRound_Cont1
 sub.w #360,P2_Dir
 bra		PlayRound_Cont1			; and jump


PR_Right:
	tst.w	FLAG_360			; check if we're playing in 4key mode
	bne		PR_Right_360				; if =0, then jump

	tst.w	KST_RIGHT				; check if RIGHT has been released since last call
	bne		PlayRound_Cont1			; if not, jump
	move.w	#1,KST_RIGHT			; else set RIGHT keystate
	
	tst.w	FLAG_TwoKey			; check if we're playing in 4key mode
	bne		PlayRound_Cont1			; if =0, then jump
	
	cmp.w	#dir_left,P2_Dir  		; check if we'd kill ourselves if we did this (by going
									; back on ourselves
	beq		PlayRound_Cont1			; if so, ignore
	
	move.w	#dir_right,P2_Dir		; otherwise, set the direction
	bra		PlayRound_Cont1			; and jump

PR_Right_360:
 sub.w #10,P2_Dir
 bge PlayRound_Cont1
 add.w #360,P2_Dir
	bra		PlayRound_Cont1			; and jump

PR_2nd:
	tst.w FLAG_360
	bne PR_2nd_360

	tst.w	KST_2ND 				; check if DOWN has been released since last call
	bne		PlayRound_Cont2			; if not, jump
	move.w	#1,KST_2ND				; else set DOWN keystate
	
	tst.w	FLAG_FourKey			; check if we're playing in 4key mode
	beq		PR_2nd_2Key				; if =0, then jump
	
	cmp.w	#dir_down,P1_Dir  		; check if we'd kill ourselves if we did this (by going
									; back on ourselves
	beq		PlayRound_Cont2			; if so, ignore
	
	move.w	#dir_up,P1_Dir			; otherwise, set the direction
	bra		PlayRound_Cont2			; and jump
	
PR_2nd_2Key:
    add.w   #3,P1_Dir				; basically:  P1_Dir=(P1_Dir+3) mod 4
    move.w  P1_Dir,d2
    move.w  P1_Dir,d1
    lsr.w   #2,d2
    lsl.w   #2,d2
    sub.w   d2,d1
    move.w  d1,P1_Dir
PR_2nd_360:	
	bra		PlayRound_Cont2			; and jump

	
PR_Home:
	tst.w FLAG_360
	bne PR_Home_360

	tst.w	KST_HOME				; check if HOME has been released since last call
	bne		PlayRound_Cont2			; if not, jump
	move.w	#1,KST_HOME				; else set HOME keystate
	
	tst.w	FLAG_FourKey			; check if we're playing in 4key mode
	beq		PR_Home_2Key			; if =0, then jump
	
	cmp.w	#dir_up,P1_Dir			; check if we'd kill ourselves if we did this (by going
									; back on ourselves
	beq		PlayRound_Cont2			; if so, ignore
	
	move.w	#dir_down,P1_Dir		; otherwise, set the direction
	bra		PlayRound_Cont2			; and jump
	
PR_Home_2Key:
    add.w   #1,P1_Dir				; basically:  P1_Dir=(P1_Dir+1) mod 4
    move.w  P1_Dir,d2
    move.w  P1_Dir,d1
    lsr.w   #2,d2
    lsl.w   #2,d2
    sub.w   d2,d1
    move.w  d1,P1_Dir
PR_Home_360:	
	bra		PlayRound_Cont2			; and jump
	
PR_Diamond:
	tst.w FLAG_360
	bne PR_Diamond_360

	tst.w	KST_DIAMOND				; check if LEFT has been released since last call
	bne		PlayRound_Cont2			; if not, jump
	move.w	#1,KST_DIAMOND			; else set LEFT keystate
	
	tst.w	FLAG_FourKey			; check if we're playing in 4key mode
	beq		PlayRound_Cont2			; if =0, then jump
	
	cmp.w	#dir_right,P1_Dir  		; check if we'd kill ourselves if we did this (by going
									; back on ourselves
	beq		PlayRound_Cont2			; if so, ignore
	
	move.w	#dir_left,P1_Dir		; otherwise, set the direction
	bra		PlayRound_Cont2			; and jump

PR_Diamond_360:
 add.w #10,P1_Dir
 cmp #360,P1_Dir
 blt PlayRound_Cont2
 sub.w #360,P1_Dir
 bra		PlayRound_Cont2			; and jump

PR_Alpha:
	tst.w FLAG_360
	bne PR_Alpha_360

	tst.w	KST_ALPHA				; check if RIGHT has been released since last call
	bne		PlayRound_Cont2			; if not, jump
	move.w	#1,KST_ALPHA			; else set RIGHT keystate
	
	tst.w	FLAG_FourKey			; check if we're playing in 4key mode
	beq		PlayRound_Cont2			; if =0, then jump
	
	cmp.w	#dir_left,P1_Dir  		; check if we'd kill ourselves if we did this (by going
									; back on ourselves
	beq		PlayRound_Cont2			; if so, ignore
	
	move.w	#dir_right,P1_Dir		; otherwise, set the direction
	bra		PlayRound_Cont2			; and jump

PR_Alpha_360:
 sub.w #10,P1_Dir
 bge PlayRound_Cont2
 add.w #360,P1_Dir
	bra		PlayRound_Cont2			; and jump

	
PlayRound_Crash:
	tst.w	P1_CrashFlag			; test if player 1 crashed
	beq		PlayRound_Crash_Cont1	; if =0 then jump
	add.w	#1,P2_Score				; else give the other player a point
PlayRound_Crash_Cont1:
	tst.w	P2_CrashFlag			; test if player 2 crashed
	beq		PlayRound_Crash_Cont2	; if =0 then jump
	add.w	#1,P1_Score				; else give the other player a point
PlayRound_Crash_Cont2:
	bsr		DrawScores				; update scores	

	tst.w	P1_CrashFlag			; test if player 1 crashed
	beq		PlayRound_Crash_Cont3	; if =0 then jump
	
	move.w	#1,d0					; fill the area with black
	move.w	#1,d1   
	move.w	#25,d2
	move.w	#6,d3
	move.w	#2,d4
	jsr		graphlib::fill	

	move.w	#0,-(a7)				; show CRASH! on their side of the screen
	pea		STR_Crash
	move.w	#2,-(a7)
	move.w	#2,-(a7)
	jsr		doorsos::DrawStrXY
	add.l	#10,a7
	
	
PlayRound_Crash_Cont3:

	tst.w	P2_CrashFlag			; test if player 2 crashed
	beq		PlayRound_Crash_Cont4	; if =0 then jump

	move.w	#133,d0					; fill the area with black
	move.w	#1,d1   
	move.w	#25,d2
	move.w	#6,d3
	move.w	#2,d4
	jsr		graphlib::fill	

	move.w	#0,-(a7)				; show CRASH! on their side of the screen
	pea		STR_Crash
	move.w	#2,-(a7)
	move.w	#135,-(a7)
	jsr		doorsos::DrawStrXY
	add.l	#10,a7
	

PlayRound_Crash_Cont4:
	move.w	#0,d0					; return 0 (round complete)
	rts
PlayRound_Esc:
	move.w	#0,teacherkeyload		; set that the teacher key wasnt what quit
	move.w	#1,d0					; return 1 (ESC pressed)
	rts
PlayRound_F1:
	move.w	#1,teacherkeyload		; set that the teacher key was what quit, so you have to keep
									; playing w/ the same scores next time
	move.w	#2,d0					; return 2 (F1 pressed)
	rts
	
; ** Updates positions of and draws the 2 players' dots and checks for collisions
;	 OUTPUT:	[P1_CrashFlag],[P2_CrashFlag]
DrawDots_360:
 move.w P1_X,d0
 move.w P1_Y,d1
 move.w P1_Dir,d2
 bsr moveangle
 move.w d0,d3
 move.w d1,d4
 clr.w d5
 lsr.w #8,d3
 addx.w d5,d3
 lsr.w #8,d4
 addx.w d5,d4
 tst.w	FLAG_WrapAround			; check if playing with WrapAround option
 beq		DrawDots_Continue_360		; if not, don't worry about it
 cmp.w	#99,d4					; if so, check if we need to wrap this time
 bne		DrawDots_360_1		; if not, don't worry about it
 cmp.w #98,P1_LY
 bne DrawDots_360_1
 move.w	#2304,d1					; if so, wrap
 bra		DrawDots_Continue_360		; and jump
DrawDots_360_1:
 cmp.w	#1,d3					; if so, check if we need to wrap this time
 bne		DrawDots_360_2		; if not, don't worry about it
 cmp.w	#2,P1_LX					; if so, check if we need to wrap this time
 bne		DrawDots_360_2		; if not, don't worry about it
 move.w	#40448,d0				; if so, wrap
 bra		DrawDots_Continue_360		; and jump
DrawDots_360_2:
 cmp.w	#159,d3					; if so, check if we need to wrap this time
 bne		DrawDots_360_3		; if not, don't worry about it
 cmp.w	#158,P1_LX					; if so, check if we need to wrap this time
 bne		DrawDots_360_3		; if not, don't worry about it
 move.w	#256,d0				; if so, wrap
 bra		DrawDots_Continue_360		; and jump
DrawDots_360_3:
 cmp.w	#8,d4					; if so, check if we need to wrap this time
 bne		DrawDots_Continue_360		; if not, don't worry about it
 cmp.w	#9,P1_LY					; if so, check if we need to wrap this time
 bne		DrawDots_Continue_360		; if not, don't worry about it
 move.w	#25088,d1				; if so, wrap
 bra		DrawDots_Continue_360		; and jump
DrawDots_Continue_360:
 move.w d0,P1_X
 move.w d1,P1_Y
;
 move.w P2_X,d0
 move.w P2_Y,d1
 move.w P2_Dir,d2
 bsr moveangle
 move.w d0,d3
 move.w d1,d4
 clr.w d5
 lsr.w #8,d3
 addx.w d5,d3
 lsr.w #8,d4
 addx.w d5,d4
 tst.w	FLAG_WrapAround			; check if playing with WrapAround option
 beq		DrawDots_360_Test1		; if not, don't worry about it
 cmp.w	#99,d4					; if so, check if we need to wrap this time
 bne		DrawDots_360_4		; if not, don't worry about it
 cmp.w #98,P2_LY
 bne DrawDots_360_1
 move.w	#2304,d1					; if so, wrap
 bra		DrawDots_360_Test1		; and jump
DrawDots_360_4:
 cmp.w	#1,d3					; if so, check if we need to wrap this time
 bne		DrawDots_360_5		; if not, don't worry about it
 cmp.w	#2,P2_LX					; if so, check if we need to wrap this time
 bne		DrawDots_360_5		; if not, don't worry about it
 move.w	#40448,d0				; if so, wrap
 bra		DrawDots_360_Test1		; and jump
DrawDots_360_5:
 cmp.w	#159,d3					; if so, check if we need to wrap this time
 bne		DrawDots_360_6		; if not, don't worry about it
 cmp.w	#158,P2_LX					; if so, check if we need to wrap this time
 bne		DrawDots_360_6		; if not, don't worry about it
 move.w	#256,d0				; if so, wrap
 bra		DrawDots_360_Test1		; and jump
DrawDots_360_6:
 cmp.w	#8,d4					; if so, check if we need to wrap this time
 bne		DrawDots_360_Test1		; if not, don't worry about it
 cmp.w	#9,P2_LY					; if so, check if we need to wrap this time
 bne		DrawDots_360_Test1		; if not, don't worry about it
 move.w	#25088,d1				; if so, wrap
 bra		DrawDots_360_Test1		; and jump
;
DrawDots_360_Test1:
 move.w d0,P2_X
 move.w d1,P2_Y
    move.w  P1_X,d0             ; check player 1's new pixel position for collision
    move.w  P1_Y,d1
    clr.w d3
 lsr.w #8,d0
 addx.w d3,d0
 lsr.w #8,d1
 addx.w d3,d1
 cmp P1_LX,d0
 bne DrawDots_360_Test1_2
 cmp P1_LY,d1
 beq DrawDots_360_Test2
DrawDots_360_Test1_2:
	bsr		GetPixel
	tst.w	d0						; if it equals 0 ...
	beq DrawDots_360_Test2		; ... then loop
	move.w	#1,P1_CrashFlag			; otherwise set the flag
;
DrawDots_360_Test2:
    move.w  P2_X,d0             ; check player 1's new pixel position for collision
    move.w  P2_Y,d1
    clr.w d3
 lsr.w #8,d0
 addx.w d3,d0
 lsr.w #8,d1
 addx.w d3,d1
 cmp P2_LX,d0
 bne DrawDots_360_Test2_2
 cmp P2_LY,d1
 beq DrawDots_360_Draw
DrawDots_360_Test2_2:
	bsr		GetPixel
	tst.w	d0						; if it equals 0 ...
	beq DrawDots_360_Draw		; ... then loop
	move.w	#1,P2_CrashFlag			; otherwise set the flag
;
DrawDots_360_Draw:
    move.w  P1_X,d0             ; check player 1's new pixel position for collision
    move.w  P1_Y,d1
    clr.w d3
 lsr.w #8,d0
 addx.w d3,d0
 lsr.w #8,d1
 addx.w d3,d1
 move.w d0,P1_LX
 move.w d1,P1_LY
	bsr		PixelOn
;
    move.w  P2_X,d0             ; check player 1's new pixel position for collision
    move.w  P2_Y,d1
    clr.w d3
 lsr.w #8,d0
 addx.w d3,d0
 lsr.w #8,d1
 addx.w d3,d1
 move.w d0,P2_LX
 move.w d1,P2_LY
	bsr		PixelOn
	rts								; and we're done


DrawDots:
	move.w	#0,P1_CrashFlag			; reset crash flags to 0
	move.w	#0,P2_CrashFlag

	tst.w FLAG_360
	bne DrawDots_360
	
	move.w  P1_Dir,d0           ; put player 1's direction in d0.w
	
	cmp.w	#dir_left,d0			; is player 1 going left?
	beq		P1_left					; if so, jump
	cmp.w	#dir_right,d0			; is player 1 going right?
	beq		P1_right				; if so, jump
	cmp.w	#dir_up,d0				; is player 1 going up?
	beq		P1_up					; if so, jump
	
	add.w	#1,P1_Y					; player 1 must be going down, so add 1 to Y position
	tst.w	FLAG_WrapAround			; check if playing with WrapAround option
	beq		DrawDots_Continue1		; if not, don't worry about it
	cmp.w	#99,P1_Y					; if so, check if we need to wrap this time
	bne		DrawDots_Continue1		; if not, don't worry about it
	
	move.w	#9,P1_Y					; if so, wrap
	bra		DrawDots_Continue1		; and jump
P1_left:
	sub.w	#1,P1_X					; update X position
	
	tst.w	FLAG_WrapAround			; check if playing with WrapAround option
	beq		DrawDots_Continue1		; if not, don't worry about it
	cmp.w	#0,P1_X					; if so, check if we need to wrap this time
	bne		DrawDots_Continue1		; if not, don't worry about it
	
	move.w	#158,P1_X				; if so, wrap
	bra		DrawDots_Continue1		; and jump
P1_right:
	add.w	#1,P1_X					; update X position
	tst.w	FLAG_WrapAround			; check if playing with WrapAround option
	beq		DrawDots_Continue1		; if not, don't worry about it
	cmp.w	#159,P1_X					; if so, check if we need to wrap this time
	bne		DrawDots_Continue1		; if not, don't worry about it
	
	move.w	#1,P1_X				; if so, wrap
	bra		DrawDots_Continue1		; and jump
P1_up:
	sub.w	#1,P1_Y					; update Y position
	tst.w	FLAG_WrapAround			; check if playing with WrapAround option
	beq		DrawDots_Continue1		; if not, don't worry about it
	cmp.w	#8,P1_Y					; if so, check if we need to wrap this time
	bne		DrawDots_Continue1		; if not, don't worry about it
	
	move.w	#98,P1_Y				; if so, wrap
DrawDots_Continue1:
    move.w  P2_Dir,d0           ; put player 2's direction in d0.w
	
	cmp.w	#dir_left,d0			; is player 2 going left?
	beq		P2_left					; if so, jump
	cmp.w	#dir_right,d0			; is player 2 going right?
	beq		P2_right				; if so, jump
	cmp.w	#dir_up,d0				; is player 2 going up?
	beq		P2_up					; if so, jump
	
	add.w	#1,P2_Y					; player 2 must be going down, so add 1 to Y position

	tst.w	FLAG_WrapAround			; check if playing with WrapAround option
	beq		DrawDots_Continue2		; if not, don't worry about it
	cmp.w	#99,P2_Y				; if so, check if we need to wrap this time
	bne		DrawDots_Continue2		; if not, don't worry about it
	
	move.w	#9,P2_Y					; if so, wrap
	bra		DrawDots_Continue2		; and jump
P2_left:
	sub.w	#1,P2_X					; update X position
	tst.w	FLAG_WrapAround			; check if playing with WrapAround option
	beq		DrawDots_Continue2		; if not, don't worry about it
	cmp.w	#0,P2_X 				; if so, check if we need to wrap this time
	bne		DrawDots_Continue2		; if not, don't worry about it
	
	move.w	#158,P2_X				; if so, wrap
	bra		DrawDots_Continue2		; and jump
P2_right:
	add.w	#1,P2_X					; update X position
	tst.w	FLAG_WrapAround			; check if playing with WrapAround option
	beq		DrawDots_Continue2		; if not, don't worry about it
	cmp.w	#159,P2_X				; if so, check if we need to wrap this time
	bne		DrawDots_Continue2		; if not, don't worry about it
	
	move.w	#1,P2_X					; if so, wrap
	bra		DrawDots_Continue2		; and jump
P2_up:
	sub.w	#1,P2_Y					; update Y position
	tst.w	FLAG_WrapAround			; check if playing with WrapAround option
	beq		DrawDots_Continue2		; if not, don't worry about it
	cmp.w	#8,P2_Y					; if so, check if we need to wrap this time
	bne		DrawDots_Continue2		; if not, don't worry about it
	
	move.w	#98,P2_Y				; if so, wrap
DrawDots_Continue2:
    move.w  P1_X,d0             ; check player 1's new pixel position for collision
    move.w  P1_Y,d1
	bsr		GetPixel
	tst.w	d0						; if it equals 0 ...
	beq		DrawDots_Continue3		; ... then loop
	move.w	#1,P1_CrashFlag			; otherwise set the flag
DrawDots_Continue3:
    move.w  P2_X,d0             ; check player 2's new pixel position for collision
    move.w  P2_Y,d1
	bsr		GetPixel
	tst.w	d0						; if it equals 0 ...
	beq		DrawDots_Continue4		; ... then loop
	move.w	#1,P2_CrashFlag			; otherwise set the flag
DrawDots_Continue4:
    move.w  P1_X,d0             ; draw player 1's dot
    move.w  P1_Y,d1
	bsr		PixelOn
    move.w  P2_X,d0             ; draw player 2's dot
    move.w  P2_Y,d1
	bsr		PixelOn
	rts								; and we're done

;***360 Movement Routine - by Scott Noveck <noveck@pluto.njcc.com> ***
;INPUT:
; d0 = x (shifted left 8)
; d1 = y (shifted left 8)
; d2 = angle
moveangle:
 lea sintable(pc),a0
 move.w d2,d5
 clr.w d3
moveangle_2:
 cmp.w #90,d2
 blt \move<90
 cmp.w #180,d2
 blt \move<180
 cmp.w #270,d2
 blt \move<270
 cmp.w #360,d2
 blt \move<360
 sub.w #360,d2
 bra moveangle_2
 rts
\move<90:
 move.b 0(a0,d5.w),d3
 sub.w d3,d1
 move.b #90,d4
 sub.b d5,d4
 move.b 0(a0,d4.w),d3
 add.w d3,d0
 rts
\move<180:
 sub.w #90,d5
 move.b 0(a0,d5.w),d3
 sub.w d3,d0
 move.b #90,d4
 sub.b d5,d4
 move.b 0(a0,d4.w),d3
 sub.w d3,d1
 rts
\move<270:
 sub.w #180,d5
 move.b 0(a0,d5.w),d3
 add.w d3,d1
 move.b #90,d4
 sub.b d5,d4
 move.b 0(a0,d4.w),d3
 sub.w d3,d0
 rts
\move<360:
 sub.w #270,d5
 move.b 0(a0,d5.w),d3
 add.w d3,d0
 move.b #90,d4
 sub.b d5,d4
 move.b 0(a0,d4.w),d3
 add.w d3,d1
 rts

	
; ** Sets keystates of useful keys that are released to 0
;    UP/DOWN/LEFT/RIGHT/2nd/ALPHA/DIAMOND/HOME
ClearKeyStates:
	move.w 	#kbd_arrows,d0			; read arrow row keys (UP,DOWN,LEFT,RIGHT,2nd,Diamond,Alpha)
	bsr		ReadKBD
	
	btst.w	#0,d0					; check UP
	beq		ClearKeyStates_Jump1	; if pressed, jump
	move.w	#0,KST_UP				; else clear out the flag
ClearKeyStates_Jump1:
	btst.w	#1,d0					; check LEFT
	beq		ClearKeyStates_Jump2	; if pressed, jump
	move.w	#0,KST_LEFT				; else clear out the flag
ClearKeyStates_Jump2:
	btst.w	#2,d0					; check DOWN
	beq		ClearKeyStates_Jump3	; if pressed, jump
	move.w	#0,KST_DOWN				; else clear out the flag
ClearKeyStates_Jump3:
	btst.w	#3,d0					; check RIGHT
	beq		ClearKeyStates_Jump4	; if pressed, jump
	move.w	#0,KST_RIGHT			; else clear out the flag
ClearKeyStates_Jump4:
	btst.w	#4,d0					; check 2ND
	beq		ClearKeyStates_Jump5	; if pressed, jump
	move.w	#0,KST_2ND				; else clear out the flag
ClearKeyStates_Jump5:
	btst.w	#6,d0					; check DIAMOND
	beq		ClearKeyStates_Jump6	; if pressed, jump
	move.w	#0,KST_DIAMOND			; else clear out the flag
ClearKeyStates_Jump6:
	btst.w	#7,d0					; check ALPHA
	beq		ClearKeyStates_Jump7	; if pressed, jump
	move.w	#0,KST_ALPHA			; else clear out the flag
ClearKeyStates_Jump7:
	move.w	#kbd_column1,d0			; read column 1 key (HOME)
	bsr		ReadKBD
	
	btst.w	#6,d0					; check HOME
	beq		ClearKeyStates_Jump8	; if pressed, jump
	move.w	#0,KST_HOME				; else clear out the flag
ClearKeyStates_Jump8:
	rts
	
; ** Do a loop [DelayCycles] times
GameDelay:
	move.w	#11,d5
	sub.w	GameSpeed(pc),d5		; put 11-GameSpeed (so higher #=faster) into d5
	move.w	#70,d6					; cycles= 70+30*(11-GameSpeed)
GameDelay_Multiply:
	add.w	#30,d6
	sub.w	#1,d5
	tst.w	d5
	bne		GameDelay_Multiply
		
GameDelay_Loop:
	bsr		ClearKeyStates			; do ClearKeyStates (super-fast game response)
	
	sub.w	#1,d6					; more cycles needed?
	tst.w	d6						
	bne		GameDelay_Loop			; if so, loop
	rts
	
; ***************************************************************
; ********************** Misc. subroutines **********************
; ***************************************************************

; ************ General Screen-related Routines

; ** Sets a pixel to black
;    INPUT:		d0.w=x coordinate
;				d1.w=y coordinate
PixelOn:
    clr.l   d2					; clear out upper bits of d2
    move.w  d0,d2				; and copy d0.w into it
	
    move.l  #LCD_MEM,a0			; put screen offset into a0
    lsr.w   #3,d2				; divide x coordinate by 8
    add.l   d2,a0    			; and add it to the screen offset
	
    lsl.w   #3,d2				; get modulo of x coordinate and 8
    sub.w   d2,d0
	
    move.w  #7,d3				; 7-(x mod 8) => d3 (for later)
    sub.w   d0,d3
	
PixelOn_Loop1:					; add 30 bytes for every Y row to skip
    tst.w   d1
    beq     PixelOn_Continue
    add.l   #30,a0
    sub.w   #1,d1
    bra     PixelOn_Loop1
	
PixelOn_Continue:
    move.b  (a0),d5				; get the 8 pixels currently in byte we need to change
    bset    d3,d5				; and set the correct bit
    move.b  d5,(a0)				; and put it back onto the screen
	
    rts

; ** Gets the value of a pixel
;    INPUT:		d0.w=x coordinate
;				d1.w=y coordinate
; 	 OUTPUT:	d0.w=pixel state (0=off, 1=on)
GetPixel:
    clr.l   d2					; clear out upper bits of d2
    move.w  d0,d2				; and copy d0.w into it
	
    move.l  #LCD_MEM,a0			; put screen offset into a0
    lsr.w   #3,d2				; divide x coordinate by 8
    add.l   d2,a0    			; and add it to the screen offset
	
    lsl.w   #3,d2				; get modulo of x coordinate and 8
    sub.w   d2,d0
	
    move.w  #7,d3				; 7-(x mod 8) => d3 (for later)
    sub.w   d0,d3
	
GetPixel_Loop1:					; add 30 bytes for every Y row to skip
    tst.w   d1
    beq     GetPixel_Continue
    add.l   #30,a0
    sub.w   #1,d1
    bra     GetPixel_Loop1
	
GetPixel_Continue:
    move.b  (a0),d5				; get the 8 pixels currently in byte we need to check
    btst.b  d3,d5				; and check it
	beq		GetPixel_RetOff	    ; if 0, jump
	move.w	#1,d0				; return 1
	rts
GetPixel_RetOff:
	move.w	#0,d0				; return 0
	rts

; ** Clears d0.w lines from a0.l
;	 INPUT: 	a0.l=screen offset to start at
;	 			d0.w=number of lines to clear (*MUST* be at least 1)
ClearLines:
	move.l	#00,(a0)+
	move.l	#00,(a0)+
	move.l	#00,(a0)+
	move.l	#00,(a0)+
	move.l	#00,(a0)+
	add.l	#10,a0
	sub.w	#1,d0
	tst.w	d0
	bne		ClearLines
	rts
	
; ** Copies d0.w lines from a0.l to a1.l
;	 INPUT: 	a0.l=source screen offset
;				a1.l=destination screen offset
;	 			d0.w=number of lines to copy (*MUST* be at least 1)
CopyLines:
	move.l	(a0)+,(a1)+
	move.l	(a0)+,(a1)+
	move.l	(a0)+,(a1)+
	move.l	(a0)+,(a1)+
	move.l	(a0)+,(a1)+
	add.l	#10,a0
	add.l	#10,a1
	sub.w	#1,d0
	tst.w	d0
	bne		CopyLines
	rts
	
	
; ** Clear out screen buffer at a0.l
;	 INPUT:		a0.l=screen offset
ClearScreen:
    move.l  d0,-(a7)                    ; save d0
    move.w  #100,d0
    bsr     ClearLines
    move.l  (a7)+,d0                    ; restore d0
    rts
	
; ** Set the screen font to SMALL
SetFontSMALL:
    move.w  #0,-(a7)
    jsr     doorsos::FontSetSys
    add.l   #2,a7
    rts

; ************ General Keyboard Routines

; ** Read keyboard directly
;    INPUT:  	d0.w=keyboard mask
;    OUTPUT:	d0.b=keys hit
ReadKBD:
	move.w	d1,-(a7)
	move.w	d0,d1
	move.w  #$700,d0                    ; Disable interrupts, saving old in mask in d0
    trap    #1

	move.w	d1,($600018)			; set keymask
  	nop								; pause so TI89 has time to apply the mask
  	nop
  	nop
  	nop
	nop
  	nop
  	nop
  	nop
  	nop
  	nop
	nop
  	nop
	nop
  	nop
	nop
  	nop
	clr.w	d1
  	move.b	($60001b),d1			; get keys hit
	trap	#1
	move.w	d1,d0  	
	move.w	(a7)+,d1
	rts

; ** Wait for all buttons to be released
WaitKBD:
	move.w	#$00,d0					; set keymask to read all keys
	bsr		ReadKBD
	cmp.b	#$ff,d0					; check to see if any key is pressed
	bne		WaitKBD					; if so, start over
	rts

; ** Pause program (like util::idle_loop but works directly)
Pause:
	bsr		WaitKBD					; wait for all buttons to be released
Pause_Loop1:
	move.w	#$00,d0					; set keymask to read all keys
	bsr		ReadKBD
	cmp.w	#$ff,d0					; check to see if any key is pressed
	beq		Pause_Loop1				; if not, loop
	bsr		WaitKBD					; wait for all buttons to be released
	rts
	
; ***************************************************************
; ********************** Interrupts section *********************
; ***************************************************************
New_Int1:							; empty INT 1
	rte

; ***************************************************************
; ************************* Data section ************************
; ***************************************************************

titlePicBits:
    incbin "stron89.huf"              ; compressed picture data

Old_Int1		dc.l 0

; ** Teacher key stuff
teacherkeyload	dc.w	0

; ** Game Variables that may retain between games (b/c of teacher key)
P1_Score  		dc.w    0

P2_Score  		dc.w    0

; ** Option Variables
GameSpeed       dc.w    5

; ** Option Flags
FLAG_WrapAround dc.w    0
FLAG_FourKey	dc.w	1
FLAG_TwoKey	dc.w	0
FLAG_360	dc.w	0

; ** Strings
STR_GetReady	dc.b "GET READY...",0
STR_Crash		dc.b "CRASH!",0

STR_mask1   	dc.b "%i",0			; used to format and display scores
STR_mask2   	dc.b "0%s",0			; used to format and display scores

STR_info1       dc.b "Super Tron 89 v1.2 - timbavaro@hotmail.com",0

STR_option1		dc.b "SPEED",0
STR_option2		dc.b "SCREEN WRAP",0
STR_option3		dc.b "CONTROL STYLE",0
STR_option4		dc.b "EXIT",0

STR_option2_0	dc.b "OFF",0
STR_option2_1	dc.b "ON",0

STR_option3_0	dc.b "2-KEY",0
STR_option3_1	dc.b "4-KEY",0
STR_option3_2	dc.b " 360",0

sintable:
 dc.b 001,004,009,013,018,022,027,031,035,040	;00 - 09
 dc.b 044,049,053,057,062,066,070,075,079,083	;10 - 19
 dc.b 087,091,096,100,104,108,112,116,120,124	;20 - 29
 dc.b 128,131,135,139,143,146,150,153,157,160	;30 - 39
 dc.b 164,167,171,174,177,180,183,186,190,192	;40 - 49
 dc.b 195,198,201,204,206,209,211,214,216,219	;50 - 59
 dc.b 221,223,225,227,229,231,233,235,236,238	;60 - 69
 dc.b 240,241,243,244,245,246,247,248,249,250	;70 - 79
 dc.b 251,252,253,253,254,254,254,255,255,255	;80 - 89
 dc.b 255,001					;90,91 -- just in case =)

; ** Comment
_comment    dc.b "Super Tron 89 v1.2",0

        bss
screensave0 ds.b 2000				; area for decompressed picture data (for title screen)
screensave1 ds.b 2000

; ** Buffers
buffer1 ds.b    16					; small misc. buffer
buffer2 ds.b    16					; small misc. buffer

; ** Game variables
P1_X   			dc.w    0
P1_Y   			dc.w    0 

P2_X   			dc.w    0
P2_Y   			dc.w    0

P1_Dir 			dc.w    0
P1_CrashFlag  	dc.w    0

P2_Dir 	    	dc.w    0
P2_CrashFlag  	dc.w    0

; ** Keystates
KST_UP			dc.w	0
KST_DOWN  		dc.w	0
KST_LEFT		dc.w	0
KST_RIGHT  		dc.w	0

KST_2ND			dc.w	0
KST_DIAMOND		dc.w	0
KST_ALPHA		dc.w	0
KST_HOME  		dc.w	0

P1_LX			dc.w 0
P1_LY			dc.w 0
P2_LX			dc.w 0
P2_LY			dc.w 0

; ** Game Menu Variables
barpos			dc.b 0				; position of Game Menu selection bar {0,1,2}
bigbarpos		dc.b 0				; position of Option Menu selection bar {0,1,2,3}

        end

